import unittest

def invalid_exc(func, *arg, **kw):
    from ptah.form import Invalid
    try:
        func(*arg, **kw)
    except Invalid, e:
        return e
    else:
        raise AssertionError('Invalid not raised') # pragma: no cover


class DummyValidator(object):
    def __init__(self, msg=None):
        self.msg = msg

    def __call__(self, node, value):
        from ptah.form import Invalid
        if self.msg:
            raise Invalid(node, self.msg)


class TestAll(unittest.TestCase):
    def _makeOne(self, validators):
        from ptah.form import All
        return All(*validators)

    def test_success(self):
        validator1 = DummyValidator()
        validator2 = DummyValidator()
        validator = self._makeOne([validator1, validator2])
        self.assertEqual(validator(None, None), None)

    def test_failure(self):
        validator1 = DummyValidator('msg1')
        validator2 = DummyValidator('msg2')
        validator = self._makeOne([validator1, validator2])
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, ['msg1', 'msg2'])


class TestFunction(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from ptah.form import Function
        return Function(*arg, **kw)

    def test_success_function_returns_True(self):
        validator = self._makeOne(lambda x: True)
        self.assertEqual(validator(None, None), None)

    def test_fail_function_returns_empty_string(self):
        validator = self._makeOne(lambda x: '')
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'Invalid value')

    def test_fail_function_returns_False(self):
        validator = self._makeOne(lambda x: False)
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'Invalid value')

    def test_fail_function_returns_string(self):
        validator = self._makeOne(lambda x: 'fail')
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'fail')

    def test_propagation(self):
        validator = self._makeOne(lambda x: 'a' in x, 'msg')
        self.assertRaises(TypeError, validator, None, None)

class TestRange(unittest.TestCase):
    def _makeOne(self, **kw):
        from ptah.form import Range
        return Range(**kw)

    def test_success_no_bounds(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, 1), None)

    def test_success_upper_bound_only(self):
        validator = self._makeOne(max=1)
        self.assertEqual(validator(None, -1), None)

    def test_success_minimum_bound_only(self):
        validator = self._makeOne(min=0)
        self.assertEqual(validator(None, 1), None)

    def test_success_min_and_max(self):
        validator = self._makeOne(min=1, max=1)
        self.assertEqual(validator(None, 1), None)

    def test_min_failure(self):
        validator = self._makeOne(min=1)
        e = invalid_exc(validator, None, 0)
        self.assertEqual(e.msg.interpolate(), '0 is less than minimum value 1')

    def test_min_failure_msg_override(self):
        validator = self._makeOne(min=1, min_err='wrong')
        e = invalid_exc(validator, None, 0)
        self.assertEqual(e.msg, 'wrong')

    def test_max_failure(self):
        validator = self._makeOne(max=1)
        e = invalid_exc(validator, None, 2)
        self.assertEqual(e.msg.interpolate(),
                         '2 is greater than maximum value 1')

    def test_max_failure_msg_override(self):
        validator = self._makeOne(max=1, max_err='wrong')
        e = invalid_exc(validator, None, 2)
        self.assertEqual(e.msg, 'wrong')

class TestRegex(unittest.TestCase):
    def _makeOne(self, pattern):
        from ptah.form import Regex
        return Regex(pattern)

    def test_valid_regex(self):
        self.assertEqual(self._makeOne('a')(None, 'a'), None)
        self.assertEqual(self._makeOne('[0-9]+')(None, '1111'), None)
        self.assertEqual(self._makeOne('')(None, ''), None)
        self.assertEqual(self._makeOne('.*')(None, ''), None)

    def test_invalid_regexs(self):
        from ptah.form import Invalid
        self.assertRaises(Invalid, self._makeOne('[0-9]+'), None, 'a')
        self.assertRaises(Invalid, self._makeOne('a{2,4}'), None, 'ba')

    def test_regex_not_string(self):
        from ptah.form import Invalid
        import re
        regex = re.compile('[0-9]+')
        self.assertEqual(self._makeOne(regex)(None, '01'), None)
        self.assertRaises(Invalid, self._makeOne(regex), None, 't')


class TestEmail(unittest.TestCase):
    def _makeOne(self):
        from ptah.form import Email
        return Email()

    def test_valid_emails(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, 'me@here.com'), None)
        self.assertEqual(validator(None, 'me1@here1.com'), None)
        self.assertEqual(validator(None, 'name@here1.us'), None)
        self.assertEqual(validator(None, 'name@here1.info'), None)
        self.assertEqual(validator(None, 'foo@bar.baz.biz'), None)

    def test_empty_email(self):
        validator = self._makeOne()
        e = invalid_exc(validator, None, '')
        self.assertEqual(e.msg, 'Invalid email address')

    def test_invalid_emails(self):
        validator = self._makeOne()
        from ptah.form import Invalid
        self.assertRaises(Invalid, validator, None, 'me@here.')
        self.assertRaises(Invalid, validator, None, 'name@here.comcom')
        self.assertRaises(Invalid, validator, None, '@here.us')
        self.assertRaises(Invalid, validator, None, '(name)@here.info')

class TestLength(unittest.TestCase):
    def _makeOne(self, min=None, max=None):
        from ptah.form import Length
        return Length(min=min, max=max)

    def test_success_no_bounds(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, ''), None)

    def test_success_upper_bound_only(self):
        validator = self._makeOne(max=1)
        self.assertEqual(validator(None, 'a'), None)

    def test_success_minimum_bound_only(self):
        validator = self._makeOne(min=0)
        self.assertEqual(validator(None, ''), None)

    def test_success_min_and_max(self):
        validator = self._makeOne(min=1, max=1)
        self.assertEqual(validator(None, 'a'), None)

    def test_min_failure(self):
        validator = self._makeOne(min=1)
        e = invalid_exc(validator, None, '')
        self.assertEqual(e.msg.interpolate(), 'Shorter than minimum length 1')

    def test_max_failure(self):
        validator = self._makeOne(max=1)
        e = invalid_exc(validator, None, 'ab')
        self.assertEqual(e.msg.interpolate(), 'Longer than maximum length 1')

class TestOneOf(unittest.TestCase):
    def _makeOne(self, values):
        from ptah.form import OneOf
        return OneOf(values)

    def test_success(self):
        validator = self._makeOne([1])
        self.assertEqual(validator(None, 1), None)

    def test_failure(self):
        validator = self._makeOne([1, 2])
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg.interpolate(), '"None" is not one of 1, 2')



from ptah import config
from base import Base


class TestUIAction(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestUIAction, self).tearDown()

    def _setup_ptah(self):
        pass

    def test_uiaction(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'action1', 'Action 1')
        self._init_ptah()

        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)

        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['id'], 'action1')

    def test_uiaction_conflicts(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'action1', 'Action 1')
        ptah.cms.uiaction(Content, 'action1', 'Action 1')
        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_uiaction_url(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'action1', 'Action 1',
                          action='test.html')
        self._init_ptah()
        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(actions[0]['url'], 'http://localhost:8080/test.html')

    def test_uiaction_absolute_url(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'action1', 'Action 1',
                          action='/content/about.html')
        self._init_ptah()
        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(actions[0]['url'],
                         'http://localhost:8080/content/about.html')

    def test_uiaction_custom_url(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        def customAction(content, request):
            return 'http://github.com/ptahproject'

        ptah.cms.uiaction(Content, 'action1', 'Action 1',
                          action=customAction)
        self._init_ptah()
        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(actions[0]['url'], 'http://github.com/ptahproject')

    def test_uiaction_condition(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        allow = False
        def condition(content, request):
            return allow

        ptah.cms.uiaction(
            Content, 'action1', 'Action 1',
            action='test.html', condition=condition)
        self._init_ptah()
        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(len(actions), 0)

        allow = True
        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(len(actions), 1)

    def test_uiaction_permission(self):
        import ptah, ptah.cms

        class Content(object):
            __name__ = ''

        allow = False
        def check_permission(permission, content, request=None, throw=False):
            return allow

        ptah.cms.uiaction(
            Content, 'action1', 'Action 1', permission='View')
        self._init_ptah()
        request = self._makeRequest()

        orig_cp = ptah.check_permission
        ptah.check_permission = check_permission

        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(len(actions), 0)

        allow = True
        actions = ptah.cms.list_uiactions(Content(), request)
        self.assertEqual(len(actions), 1)

        ptah.check_permission = orig_cp

    def test_uiaction_sort_weight(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'view', 'View', sort_weight=1.0)
        ptah.cms.uiaction(Content, 'action', 'Action', sort_weight=2.0)
        self._init_ptah()

        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)

        self.assertEqual(actions[0]['id'], 'view')
        self.assertEqual(actions[1]['id'], 'action')

    def test_uiaction_userdata(self):
        import ptah.cms

        class Content(object):
            __name__ = ''

        ptah.cms.uiaction(Content, 'view', 'View', testinfo='test')
        self._init_ptah()

        request = self._makeRequest()

        actions = ptah.cms.list_uiactions(Content(), request)

        self.assertEqual(actions[0]['data'], {'testinfo': 'test'})



""" Node implementation """
import ptah
import sqlahelper
import sqlalchemy as sqla
from zope import interface
from collections import OrderedDict

from ptah.cms import action
from permissions import View
from interfaces import NotFound, Forbidden
from interfaces import INode, IApplicationPolicy

Base = sqlahelper.get_base()
Session = sqlahelper.get_session()


class Node(Base):
    """ Base class for persistent objects.

    .. attribute:: __uri__

       Unique object id. **Required**

    .. attribute:: __type__

       Type information object :py:class:`ptah.cms.TypeInformation`

    .. attribute:: __parent__

       Parent of node. Ptah doesn't load `__parent__` automatically.
       To load node parents use :py:func:`ptah.cms.load_parents` function.

    .. attribute:: __owner__

       URI of owner principal. It possible to load principal object
       by using :py:func:`ptah.resolve` function.

    .. attribute:: __local_roles__

       :py:class:`ptah.JsonDictType` which contains a principal uri as a
       key and a sequence of role's granted to principal for Node.

    .. attribute:: __acls__

       a :py:class:`ptah.JsonListType` of :py:class:`ptah.ACL` strings
       registered with security machinery.

    .. attribute:: __uri_factory__

       function which will return value for __uri__.  the uri must be
       resolvable by using :py:func:`ptah.resolve` function.

    """

    interface.implements(INode,
                         ptah.IACLsAware,
                         ptah.IOwnersAware,
                         ptah.ILocalRolesAware)

    __tablename__ = 'ptah_nodes'

    __id__ = sqla.Column('id', sqla.Integer, primary_key=True)
    __type_id__ = sqla.Column('type', sqla.String, info={'uri':True})
    __type__ = None

    __uri__ = sqla.Column('uri', sqla.String, unique=True,
                          nullable=False, info={'uri':True})
    __parent_uri__ = sqla.Column('parent',
                                 sqla.String,sqla.ForeignKey(__uri__),
                                 info={'uri': True})

    __owner__ = sqla.Column('owner', sqla.String, default='', info={'uri':True})
    __local_roles__ = sqla.Column('roles', ptah.JsonDictType(), default={})
    __acls__ = sqla.Column('acls', ptah.JsonListType(), default=[])

    __children__ = sqla.orm.relationship(
        'Node',
        backref=sqla.orm.backref('__parent_ref__',remote_side=[__uri__]))

    __mapper_args__ = {'polymorphic_on': __type_id__}

    __parent__ = None
    __uri_factory__ = None

    __acl__ = ptah.ACLsProperty()

    def __init__(self, **kw):
        self.__owners__ = []
        self.__local_roles__ = {}
        self.__permissions__ = []

        for attr, value in kw.items():
            setattr(self, attr, value)

        if '__parent__' in kw and kw['__parent__'] is not None:
            self.__parent_uri__ = kw['__parent__'].__uri__

        try:
            self.__uri__ = self.__uri_factory__()
        except TypeError:
            raise TypeError(
                'Subclass of Node has to override __uri_factory__')

    @action(permission=View)
    def info(self):
        info = OrderedDict(
            (('__type__', self.__type_id__),
             ('__content__', False),
             ('__uri__', self.__uri__),
             ('__parents__', [p.__uri__ for p in load_parents(self)]),
             ))

        return info


def load(uri, permission=None):
    """ Load node by `uri` and initialize __parent__ attributes. Also checks
    permission if permissin is specified.

    :param uri: Node uri
    :param permission: Check permission on node object
    :type permission: Permission id or None
    :raise KeyError: Node with this uri is not found.
    :raise Forbidden: If current principal doesn't pass permission check on loaded node.
    """
    item = ptah.resolve(uri)

    if item is not None:
        load_parents(item)

        if permission is not None:
            if not ptah.check_permission(permission, item):
                raise Forbidden()
    else:
        raise NotFound(uri)

    return item


def load_parents(node):
    """ Load and initialize `__parent__` attribute for node.
    Returns list of loaded parents.

    :param node: ptah.cms.Node node
    """
    parents = []
    parent = node
    while parent is not None:
        if not isinstance(parent, Node):
            break

        if parent.__parent__ is None:
            parent.__parent__ = parent.__parent_ref__

        parent = parent.__parent__
        if parent is not None:
            parents.append(parent)

    root = node if not parents else parents[-1]
    if not IApplicationPolicy.providedBy(root):
        try:
            root.__parent__ = get_policy()
        except AttributeError:
            # __parent__ is read onl
            pass

    return [p for p in parents if isinstance(p, Node)]


KEY = '__ptahcms_policy__'

def get_policy():
    return ptah.tldata.get(KEY)

def set_policy(policy):
    return ptah.tldata.set(KEY, policy)



from base import Base


class TestNameSchema(Base):

    def test_schema_name_validator(self):
        from ptah.cms.interfaces import ContentNameSchema

        fieldset = ContentNameSchema.bind(
            params = {'__name__': '/asdfasdfadf'})

        data, errors = fieldset.extract()
        self.assertTrue(errors)

        fieldset = ContentNameSchema.bind(
            params = {'__name__': 'asdfasdf/adf'})

        data, errors = fieldset.extract()
        self.assertTrue(errors)

        fieldset = ContentNameSchema.bind(
            params = {'__name__': ' asdfasdfadf'})

        data, errors = fieldset.extract()
        self.assertTrue(errors)



""" role """
from zope import interface
from ptah import config
from pyramid.security import Allow, Deny, ALL_PERMISSIONS, DENY_ALL
from pyramid.security import NO_PERMISSION_REQUIRED
from pyramid.httpexceptions import HTTPForbidden

from base import Base


class TestPermission(Base):

    def test_permission_register(self):
        import ptah

        perm = ptah.Permission('perm', 'Permission', 'Test permission')
        self._init_ptah()

        self.assertTrue(perm == 'perm')
        self.assertTrue(perm.title == 'Permission')
        self.assertTrue(perm.description == 'Test permission')
        self.assertTrue(ptah.get_permissions()['perm'] is perm)

    def test_permission_register_same_name(self):
        import ptah

        ptah.Permission('perm', 'Permission1')
        ptah.Permission('perm', 'Permission2')

        self.assertRaises(config.ConflictError, self._init_ptah)


class TestACL(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestACL, self).tearDown()

    def test_acl_register(self):
        import ptah

        pmap = ptah.ACL('map', 'ACL', 'Map')
        self._init_ptah()

        self.assertTrue(pmap.name == 'map')
        self.assertTrue(pmap.title == 'ACL')
        self.assertTrue(pmap.description == 'Map')
        self.assertTrue(ptah.get_acls()['map'] is pmap)

    def test_acl_register_same_name(self):
        import ptah

        ptah.ACL('map', 'acl1')
        ptah.ACL('map', 'acl2')

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_acl_allow(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow(role, 'perm1')
        pmap.allow('role:test', 'perm2')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Allow)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2', 'perm1')))

    def test_acl_allow_all(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow(role, 'perm1')
        pmap.allow(role, ALL_PERMISSIONS)
        pmap.allow(role, 'perm2')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Allow)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], ALL_PERMISSIONS)

    def test_acl_deny(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.deny(role, 'perm1')
        pmap.deny('role:test', 'perm2')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Deny)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2', 'perm1')))

    def test_acl_deny_all(self):
        import ptah

        pmap = ptah.ACL('map', 'acl map')
        pmap.deny('role:test', 'perm1')
        pmap.deny('role:test', ALL_PERMISSIONS)
        pmap.deny('role:test', 'perm2')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Deny)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], ALL_PERMISSIONS)

    def test_acl_order(self):
        import ptah

        pmap = ptah.ACL('map', 'acl map')
        pmap.deny('role:test', 'perm1')
        pmap.allow('role:test', 'perm2')
        pmap.allow('role:test2', 'perm2')
        pmap.deny('role:test2', 'perm2')

        self.assertEqual(pmap[0][0], Deny)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[1][0], Allow)
        self.assertEqual(pmap[1][1], 'role:test')
        self.assertEqual(pmap[2][0], Allow)
        self.assertEqual(pmap[2][1], 'role:test2')
        self.assertEqual(pmap[3][0], Deny)
        self.assertEqual(pmap[3][1], 'role:test2')

    def test_acl_unset_allow(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow(role, 'perm1', 'perm2')
        pmap.allow('role:test2', 'perm1')

        pmap.unset(None, 'perm1')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Allow)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2',)))

    def test_acl_unset_role_allow(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow(role, 'perm1', 'perm2')
        pmap.allow('role:test2', 'perm1')

        pmap.unset(role.id, 'perm1')

        self.assertEqual(len(pmap), 2)
        self.assertEqual(pmap[0][0], Allow)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2',)))
        self.assertEqual(pmap[1][0], Allow)
        self.assertEqual(pmap[1][1], 'role:test2')
        self.assertEqual(pmap[1][2], set(('perm1',)))

    def test_acl_unset_deny(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.deny(role, 'perm1', 'perm2')
        pmap.deny('role:test2', 'perm1')

        pmap.unset(None, 'perm1')

        self.assertEqual(len(pmap), 1)
        self.assertEqual(pmap[0][0], Deny)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2',)))

    def test_acl_unset_role_deny(self):
        import ptah

        role = ptah.Role('test', 'test')

        pmap = ptah.ACL('map', 'acl map')
        pmap.deny(role, 'perm1', 'perm2')
        pmap.deny('role:test2', 'perm1')

        pmap.unset(role.id, 'perm1')

        self.assertEqual(len(pmap), 2)
        self.assertEqual(pmap[0][0], Deny)
        self.assertEqual(pmap[0][1], 'role:test')
        self.assertEqual(pmap[0][2], set(('perm2',)))
        self.assertEqual(pmap[1][0], Deny)
        self.assertEqual(pmap[1][1], 'role:test2')
        self.assertEqual(pmap[1][2], set(('perm1',)))

    def test_acl_unset_all(self):
        import ptah

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow('role:test1', 'perm1', 'perm2')
        pmap.allow('role:test2', 'perm1')
        pmap.deny('role:test1', 'perm1', 'perm2')
        pmap.deny('role:test2', ALL_PERMISSIONS)

        pmap.unset(None, ALL_PERMISSIONS)
        self.assertEqual(len(pmap), 0)

    def test_acl_unset_role_all(self):
        import ptah

        pmap = ptah.ACL('map', 'acl map')
        pmap.allow('role:test1', 'perm2')
        pmap.allow('role:test2', 'perm1')
        pmap.deny('role:test1', 'perm1', 'perm2')
        pmap.deny('role:test2', ALL_PERMISSIONS)

        pmap.unset('role:test2', ALL_PERMISSIONS)
        self.assertEqual(len(pmap), 2)
        self.assertEqual(pmap[0][0], Allow)
        self.assertEqual(pmap[0][1], 'role:test1')
        self.assertEqual(pmap[0][2], set(('perm2',)))
        self.assertEqual(pmap[1][0], Deny)
        self.assertEqual(pmap[1][1], 'role:test1')
        self.assertEqual(pmap[1][2], set(('perm1','perm2')))


class TestACLsProps(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestACLsProps, self).tearDown()

    def test_acls(self):
        import ptah

        acl1 = ptah.ACL('acl1', 'acl1')
        acl1.allow('role1', 'perm1', 'perm2')

        acl2 = ptah.ACL('acl2', 'acl2')
        acl2.deny('role1', 'perm1', 'perm2')

        self._init_ptah()

        class Content(object):
            __acl__ = ptah.ACLsProperty()

        content = Content()

        self.assertEqual(content.__acl__, ())

        content.__acls__ = ()
        self.assertEqual(content.__acl__, ())

        content.__acls__ = ('acl1',)
        self.assertEqual(list(content.__acl__),
                         [['Allow', 'role1', set(['perm2', 'perm1'])]])

        content.__acls__ = ('acl1', 'acl2',)
        self.assertEqual(list(content.__acl__),
                         [['Allow', 'role1', set(['perm2', 'perm1'])],
                          ['Deny', 'role1', set(['perm2', 'perm1'])]])

        content.__acls__ = ('acl2', 'acl1')
        self.assertEqual(list(content.__acl__),
                         [['Deny', 'role1', set(['perm2', 'perm1'])],
                          ['Allow', 'role1', set(['perm2', 'perm1'])]])


class TestRole(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestRole, self).tearDown()

    def test_role_register(self):
        import ptah

        role = ptah.Role('myrole', 'MyRole')

        self.assertTrue(role.id == 'role:myrole')
        self.assertTrue(role.name == 'myrole')
        self.assertTrue(role.title == 'MyRole')
        self.assertTrue(role.description == '')
        self.assertTrue(str(role) == 'Role<MyRole>')
        self.assertTrue(repr(role) == 'role:myrole')

    def test_role_register_conflict(self):
        import ptah

        ptah.Role('myrole', 'MyRole1')
        ptah.Role('myrole', 'MyRole2')

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_role_roles(self):
        import ptah

        role = ptah.Role('myrole', 'MyRole')
        self._init_ptah()

        self.assertTrue('myrole' in ptah.get_roles())
        self.assertTrue(ptah.get_roles()['myrole'] is role)

    def test_role_allow_permission(self):
        import ptah
        from ptah import DEFAULT_ACL

        role = ptah.Role('myrole', 'MyRole')
        role.allow('perm1', 'perm2')

        rec = DEFAULT_ACL.get(Allow, role.id)

        self.assertEqual(rec[0], Allow)
        self.assertEqual(rec[1], role.id)
        self.assertTrue('perm1' in rec[2])
        self.assertTrue('perm2' in rec[2])

    def test_role_deny_permission(self):
        import ptah
        from ptah import DEFAULT_ACL

        role = ptah.Role('myrole', 'MyRole')
        role.deny('perm1', 'perm2')

        rec = DEFAULT_ACL.get(Deny, role.id)

        self.assertEqual(rec[0], Deny)
        self.assertEqual(rec[1], role.id)
        self.assertTrue('perm1' in rec[2])
        self.assertTrue('perm2' in rec[2])

    def test_role_unset_allowed_permission(self):
        import ptah
        from ptah import DEFAULT_ACL

        role = ptah.Role('myrole', 'MyRole')
        role.allow('perm1')

        self.assertEqual(len(DEFAULT_ACL), 1)

        role.unset('perm1')
        self.assertEqual(len(DEFAULT_ACL), 0)

    def test_role_unset_denied_permission(self):
        import ptah
        from ptah import DEFAULT_ACL

        role = ptah.Role('myrole', 'MyRole')
        role.deny('perm1')

        self.assertEqual(len(DEFAULT_ACL), 1)

        role.unset('perm1')
        self.assertEqual(len(DEFAULT_ACL), 0)


class TestDefaultRoles(Base):

    def test_role_defaults(self):
        import ptah
        self._init_ptah()

        roles = list(ptah.get_roles().keys())
        roles.sort()
        roles = roles[:3]

        self.assertTrue(['Authenticated', 'Everyone', 'Owner'] == roles)
        self.assertTrue(ptah.get_roles()['Everyone'].id == 'system.Everyone')
        self.assertTrue(ptah.get_roles()['Authenticated'].id=='system.Authenticated')
        self.assertTrue(ptah.get_roles()['Owner'].id=='system.Owner')


class Content(object):

    def __init__(self, parent=None, iface=None, acl=None):
        self.__parent__ = parent
        self.__local_roles__ = {}
        if acl is not None:
            self.__acl__ = acl

        if iface:
            interface.directlyProvides(self, iface)


class TestLocalRoles(Base):

    def test_local_role_simple(self):
        from ptah import security

        content = Content(iface=security.ILocalRolesAware)

        self.assertEqual(security.get_local_roles('userid', context=content), [])

        content.__local_roles__['userid'] = ('role:test',)

        self.assertEqual(
            security.get_local_roles('userid', context=content), ['role:test'])

    def test_local_role_lineage(self):
        from ptah import security

        parent = Content(iface=security.ILocalRolesAware)
        content = Content(parent=parent, iface=security.ILocalRolesAware)

        self.assertEqual(security.get_local_roles('userid', context=content), [])

        parent.__local_roles__['userid'] = ('role:test',)

        self.assertEqual(
            security.get_local_roles('userid', context=content), ['role:test'])

    def test_local_role_lineage_multiple(self):
        from ptah import security

        parent = Content(iface=security.ILocalRolesAware)
        content = Content(parent=parent, iface=security.ILocalRolesAware)

        self.assertEqual(security.get_local_roles('userid', context=content), [])

        parent.__local_roles__['userid'] = ('role:test',)
        content.__local_roles__['userid'] = ('role:test2',)

        lr = security.get_local_roles('userid', context=content)
        lr.sort()

        self.assertTrue(lr == ['role:test', 'role:test2'])

    def test_local_role_lineage_no_localroles(self):
        from ptah import security

        parent = Content(iface=security.ILocalRolesAware)
        content = Content(parent=parent)

        self.assertEqual(security.get_local_roles('userid', context=content), [])

        parent.__local_roles__['userid'] = ('role:test',)

        self.assertEqual(
            security.get_local_roles('userid', context=content), ['role:test'])

    def test_local_role_lineage_context_from_request(self):
        from ptah import security

        class Request(object):
            content = None
            root = None

        request = Request()

        content = Content(iface=security.ILocalRolesAware)
        content.__local_roles__['userid'] = ('role:test',)

        request.root = content

        self.assertEqual(security.get_local_roles('userid', request), ['role:test'])

        content2 = Content(iface=security.ILocalRolesAware)
        content2.__local_roles__['userid'] = ('role:test2',)

        request.context = content2
        self.assertEqual(security.get_local_roles('userid', request), ['role:test2'])


class Content2(object):

    def __init__(self, parent=None, iface=None):
        self.__parent__ = parent
        self.__owner__ = ''

        if iface:
            interface.directlyProvides(self, iface)


class TestOwnerLocalRoles(Base):

    def test_owner_role_simple(self):
        from ptah import security

        content = Content2(iface=security.IOwnersAware)

        self.assertEqual(security.get_local_roles('userid', context=content), [])

        content.__owner__ = 'userid'

        self.assertEqual(
            security.get_local_roles('userid', context=content), ['system.Owner'])

    def test_owner_role_in_parent(self):
        # return owner only on current context
        from ptah import security

        parent = Content2(iface=security.IOwnersAware)
        content = Content2(parent=parent, iface=security.IOwnersAware)

        parent.__owner__ = 'user'

        self.assertEqual(security.get_local_roles('user', context=content), [])
        self.assertEqual(
            security.get_local_roles('user', context=parent), ['system.Owner'])


class TestCheckPermission(Base):

    def setUp(self):
        super(TestCheckPermission, self).setUp()
        self._init_ptah()

    def test_checkpermission_allow(self):
        import ptah

        content = Content(acl=[DENY_ALL])

        self.assertFalse(ptah.check_permission('View', content, throw=False))
        self.assertTrue(ptah.check_permission(
            NO_PERMISSION_REQUIRED, content, throw=False))

    def test_checkpermission_deny(self):
        import ptah

        content = Content(acl=[(Allow, ptah.Everyone.id, ALL_PERMISSIONS)])

        self.assertTrue(ptah.check_permission('View', content, throw=False))
        self.assertFalse(ptah.check_permission(
            ptah.NOT_ALLOWED, content, throw=False))

    def test_checkpermission_exc(self):
        import ptah

        content = Content(acl=[DENY_ALL])

        self.assertRaises(
            HTTPForbidden, ptah.check_permission, 'View', content, throw=True)

        content = Content(acl=[(Allow, ptah.Everyone.id, ALL_PERMISSIONS)])

        self.assertRaises(
            HTTPForbidden, ptah.check_permission,
            ptah.NOT_ALLOWED, content, throw=True)

    def test_checkpermission_authenticated(self):
        import ptah

        content = Content(acl=[(Allow, ptah.Authenticated.id, 'View')])

        self.assertFalse(ptah.check_permission('View', content, throw=False))

        ptah.authService.set_userid('test-user')
        self.assertTrue(ptah.check_permission('View', content, throw=False))

    def test_checkpermission_user(self):
        import ptah

        content = Content(acl=[(Allow, 'test-user', 'View')])

        ptah.authService.set_userid('test-user')
        self.assertTrue(ptah.check_permission('View', content, throw=False))

    def test_checkpermission_superuser(self):
        import ptah
        from pyramid import security

        content = Content(
            acl=[(Deny, ptah.SUPERUSER_URI, security.ALL_PERMISSIONS)])

        ptah.authService.set_userid(ptah.SUPERUSER_URI)
        self.assertTrue(ptah.check_permission('View', content))
        self.assertFalse(ptah.check_permission(ptah.NOT_ALLOWED, content))

    def test_checkpermission_local_roles(self):
        import ptah

        content = Content(
            iface=ptah.ILocalRolesAware,
            acl=[(Allow, 'role:test', 'View')])

        ptah.authService.set_userid('test-user')
        self.assertFalse(ptah.check_permission('View', content, throw=False))

        content.__local_roles__['test-user'] = ['role:test']
        self.assertTrue(ptah.check_permission('View', content, throw=False))



""" A route and 2 views for Content """

import cgi
from paste.httpserver import serve
from ptah import view, cms


view.register_route('show_models', '/show_models')

@view.pview(route='show_models')
def show_models(request):
    models = cms.Session.query(cms.Content).all()
    return cgi.escape(str(models))

@view.pview('show_info', context=cms.Content)
def show_info(context, request):
    return cgi.escape(str(context.info()))

@view.pview('list_children', context=cms.Container)
def list_children(context, request):
    out = []
    for name, child in context.items():
        if isinstance(child, cms.Container):
            href = '<a href="%slist_children">%s</a>' #XXX extra /?
            href = href % (request.resource_url(child), child.title)
        else:
            href = '<a href="%sshow_info">%s</a>'
            href = href % (request.resource_url(child), child.title)
        out.append(href)
    return '<br />'.join(out)

if __name__ == '__main__':
    """ need to point to your settings.ini file in make_wsgi_app call.
        http://localhost:8080/show_models is url dispatch function.
        http://localhost:8080/list_children is traverser on context
        $resource_url/show_info on either folder or content.
    """
    import ptah
    app = ptah.make_wsgi_app({'settings':r'./ptah.ini'})
    serve(app, host='0.0.0.0')



import ptah
import simplejson
from ptah import config
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPNotFound

from base import Base


class TestRestRegistrations(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestRestRegistrations, self).tearDown()

    def test_rest_registerService(self):
        import ptah.rest

        srv = ptah.rest.RestService('test', 'Test service')
        self._init_ptah()

        services = config.get_cfg_storage(ptah.rest.REST_ID)

        self.assertEqual(srv.name, 'test')
        self.assertIn('test', services)
        self.assertIn(srv, services.values())

    def test_rest_registerService_conflicts(self):
        import ptah.rest

        ptah.rest.RestService('test', 'Test service')
        ptah.rest.RestService('test', 'Test service2')

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_rest_registerService_action(self):
        import ptah.rest

        srv = ptah.rest.RestService('test', 'Test service')

        @srv.action('action', 'Action')
        def raction(request, *args):
            """ doc string """
            return 'test'

        self.assertIn('action', srv.actions)

        ac = srv.actions['action']
        self.assertEqual(ac.name, 'action')
        self.assertEqual(ac.title, 'Action')
        self.assertEqual(ac.description, ' doc string ')
        self.assertEqual(ac.callable, raction)
        self.assertEqual(srv(None, 'action'), 'test')
        self.assertRaises(HTTPNotFound, srv, None, 'unknown')

    def test_rest_registerService_apidoc(self):
        import ptah.rest

        ptah.rest.RestService('test', 'Test service')
        self._init_ptah()

        srv = config.get_cfg_storage(ptah.rest.REST_ID)['test']

        @srv.action('action', 'Action')
        def raction(request, *args):
            """ doc string """

        self.assertIn('apidoc', srv.actions)

        info = srv(self._makeRequest(), 'apidoc')

        self.assertEqual(info['name'], 'test')
        self.assertEqual(info['title'], 'Test service')
        self.assertEqual(len(info['actions']), 2)

        self.assertEqual(info['actions'][0]['name'], 'apidoc')
        self.assertEqual(
            info['actions'][0]['__link__'], 'http://localhost:8080/apidoc')
        self.assertEqual(info['actions'][1]['name'], 'action')
        self.assertEqual(
            info['actions'][1]['__link__'], 'http://localhost:8080/action')


class Principal(object):
    uri = 'testprincipal:1'
    name = 'admin'
    login = 'admin'


class Provider(object):
    principal = Principal()

    def authenticate(self, creds):
        return self.principal


class TestRestView(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestRestView, self).tearDown()

    def test_rest_enable_api(self):
        from ptah.rest import RestLoginRoute, RestApiRoute

        mapper = self.p_config.get_routes_mapper()

        self.assertIsNone(mapper.get_route('ptah-rest'))
        self.assertIsNone(mapper.get_route('ptah-rest-login'))

        ptah.enable_rest_api(self.p_config)

        marker = object()

        factory = mapper.get_route('ptah-rest').factory
        self.assertIs(factory, RestApiRoute)
        self.assertIs(factory(marker).request, marker)

        factory = mapper.get_route('ptah-rest-login').factory
        self.assertIs(factory, RestLoginRoute)
        self.assertIs(factory(marker).request, marker)

    def test_rest_login(self):
        from ptah.rest import Login

        request = DummyRequest()
        login = Login(request)

        self.assertIn('authentication failed', login.render())
        self.assertEqual(request.response.status, '403 Forbidden')

    def test_rest_login_success(self):
        from ptah.rest import Login
        from ptah import authentication
        self._init_ptah()

        config.get_cfg_storage(
            authentication.AUTH_PROVIDER_ID)['test'] = Provider()
        request = DummyRequest(params = {'login': 'admin', 'password': '12345'})

        login = Login(request)
        info = simplejson.loads(login.render())

        self.assertIn('auth-token', info)
        self.assertEqual(request.response.status, '200 OK')

    def test_rest_api_auth(self):
        from ptah.rest import Api, Login
        from ptah import authentication
        self._init_ptah()

        config.get_cfg_storage(
            authentication.AUTH_PROVIDER_ID)['test'] = Provider()

        request = DummyRequest(params = {'login': 'admin', 'password': '12345'})

        login = Login(request)
        info = simplejson.loads(login.render())

        request = DummyRequest(environ = {'HTTP_X_AUTH_TOKEN': 'unknown'})
        request.matchdict = {'service': 'cms', 'subpath': ()}

        api = Api(request)
        api.render()
        self.assertEqual(ptah.authService.get_userid(), None)

        token = info['auth-token']

        request = DummyRequest(environ = {'HTTP_X_AUTH_TOKEN': token})
        request.matchdict = {'service': 'cms', 'subpath': ()}

        api = Api(request)
        api.render()
        self.assertEqual(ptah.authService.get_userid(), 'testprincipal:1')


class TestRestApi(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestRestApi, self).tearDown()

    def test_rest_unknown_service(self):
        from ptah.rest import Api
        self._init_ptah()

        request = DummyRequest()
        request.matchdict = {'service': 'test', 'subpath': ()}

        api = Api(request)
        res = simplejson.loads(api.render())
        self.assertEqual(res['message'], "'test'")
        self.assertIn("KeyError: 'test'", res['traceback'])

    def test_rest_arguments(self):
        from ptah.rest import Api, REST_ID
        self._init_ptah()

        services = config.get_cfg_storage(REST_ID)

        data = []
        def service(request, action, *args):
            data[:] = [action, args]

        services['test'] = service

        request = DummyRequest()
        request.matchdict = {'service': 'test',
                             'subpath': ('action:test','1','2')}

        api = Api(request)
        api.render()

        self.assertEqual(data[0], "action")
        self.assertEqual(data[1], ('test', '1', '2'))

    def test_rest_httpexception(self):
        from ptah.rest import Api, REST_ID
        self._init_ptah()

        services = config.get_cfg_storage(REST_ID)

        def service(request, action, *args):
            raise HTTPNotFound()

        services['test'] = service

        request = DummyRequest()
        request.matchdict = {'service': 'test',
                             'subpath': ('action:test','1','2')}
        api = Api(request)
        res = simplejson.loads(api.render())
        self.assertEqual(
            res['message'], 'The resource could not be found.')

    def test_rest_response(self):
        from ptah.rest import Api, REST_ID
        self._init_ptah()

        services = config.get_cfg_storage(REST_ID)

        def service(request, action, *args):
            return HTTPNotFound()

        services['test'] = service

        request = DummyRequest(registry=self.p_config.registry)
        request.matchdict = {'service': 'test',
                             'subpath': ('action:test','1','2')}
        api = Api(request)
        res = api.render()
        self.assertIsInstance(res, HTTPNotFound)

    def test_rest_response_data(self):
        import datetime
        from ptah.rest import Api, REST_ID
        self._init_ptah()

        services = config.get_cfg_storage(REST_ID)

        def service(request, action, *args):
            return {'dt': datetime.datetime.now()}

        services['test'] = service

        request = DummyRequest()
        request.matchdict = {'service': 'test',
                             'subpath': ('action:test','1','2')}
        api = Api(request)
        res = simplejson.loads(api.render())
        self.assertIn('dt', res)



from ptah import manage, view


class RestModule(manage.PtahModule):
    """
    REST Introspector
    """

    title = 'REST Introspector'
    manage.module('rest')


class RestModuleView(view.View):
    view.pview(
        context = RestModule,
        template = view.template('ptah.manage:templates/rest.pt'))

    def update(self):
        self.appurl = self.request.application_url
        self.url = self.request.get('url', '%s/__rest__/cms/' % self.appurl)



""" interfaces """
from zope import interface
from ptah import form
from pyramid.httpexceptions import HTTPNotFound, HTTPForbidden, HTTPServerError


class CmsException(Exception):
    """ """

class Forbidden(HTTPForbidden, CmsException):
    """ something forbidden """


class NotFound(HTTPNotFound, CmsException):
    """ something not found """


class Error(HTTPServerError, CmsException):
    """ internale error """


class INode(interface.Interface):
    """ base """

    __id__ = interface.Attribute('Id')
    __uri__ = interface.Attribute('Uri')
    __type_id__ = interface.Attribute('Node type')
    __parent_uri__ = interface.Attribute('Node parent')


class IContent(interface.Interface):
    """ base interface """

    name = interface.Attribute('Name')


class IContainer(IContent):
    """ base container content """

    __path__ = interface.Attribute('traversal path')


class IApplicationPolicy(interface.Interface):
    """ application policy """


class IApplicationRoot(IContent, IContainer):
    """ application root object """

    __root_path__ = interface.Attribute('Current mount point')


class IApplicationRootFactory(interface.Interface):
    """ application root factory """

    def __call__(**kw):
        """ return ApplicationRoot object """


class ITypeInformation(interface.Interface):
    """ Content type information """

    name = interface.Attribute('Name')
    title = interface.Attribute('Title')
    description = interface.Attribute('Description')

    permission = interface.Attribute('Add permission')

    filter_content_types = interface.Attribute('Filter addable types')
    allowed_content_types = interface.Attribute('List of addable types')
    global_allow = interface.Attribute('Addable globally')

    def create(**data):
        """ construct new content instance """

    def is_allowed(container):
        """ allow create this content in container """

    def check_context(container):
        """ same as isAllowed, but raises HTTPForbidden """

    def list_types(self, container):
        """ list addable content types """


class IBlob(INode):
    """ blob """

    mimetype = interface.Attribute('Blob mimetype')
    filename = interface.Attribute('Original filename')
    data = interface.Attribute('Blob data')


class IBlobStorage(interface.Interface):
    """ blob storage """

    def add(parent, data, mimetype=None, filename=None):
        """ add blob return uri """

    def query(uri):
        """ return blob object """

    def replace(uri, data, mimetype=None, filename=None):
        """ replace existing blob """

    def remove(uri):
        """ remove blob """


ContentSchema = form.Fieldset(

    form.FieldFactory(
        'text',
        'title',
        title = 'Title'),

    form.FieldFactory(
        'textarea',
        'description',
        title = 'Description',
        missing = u''),
    )


def specialSymbols(field, appstruct):
    if '/' in appstruct:
        raise form.Invalid(field, "Names cannot contain '/'")
    if appstruct.startswith(' '):
        raise form.Invalid(field, "Names cannot starts with ' '")


ContentNameSchema = form.Fieldset(

    form.FieldFactory(
        'text',
        '__name__',
        title = 'Short Name',
        description = 'Short name is the part that shows up in '\
                            'the URL of the item.',
        missing = u'',
        validator = specialSymbols)
    )
